
#------------------------------------------------------------------
#
# Nov. 2007 - Christine Yan (cyan@cisco.com)
#
# Copyright (c) 2007 by cisco Systems, Inc.
# All rights reserved.
#
# Version-date: v1.0-11/06/2007
#
# Name: acl_lib.tcl
#
# Desc: access lib procedures
#
# This file includes acl procedures called by policy, it processes 2 kinds of
# access list: standard access list and extended access list
#
# In general, acl logic is like the following:
#
# (1). Check if an interface has an access list to be configured. 
#      From there we can know the access list number for the interface
#      we also can know the direction for the acl.
# (2). Check the global access-list configuration.
# (3). If there is an acl list existing in show run:
#      Check if the address matches the interfafce address to decide if the
#      access list is effected. 
#      The address can be source address, destination address, a host or a
#      subnet with wildcard or any.
#      a. Check if it is a standard acl, if it is a standard acl, only 
#         source address needs to be checked, which can be a host or a
#         subnet.
#      b. Check if it is an extended acl, if it is a extended acl, 
#         the destination ip address can be checked too.
#
#      The following is the way to check if the acl ip address effects the
#      ip interface for both standard acl and extended acl:
#
#      If the acl is a host address, it has to have a whole match on all
#      octets of ip address bits with the protocol permitted ip address.
#
#      If the acl is a subnet and has a wildcard, needs to check ignoring
#      bits and match bits to decide if the ip address is in the subnet 
#      the acl defined.
# (4). If there is an address match found, Check if the protocol is matched.
#      For standard acl, it is always ip protocol. 
#      For extended acl, we need to check which protocol is configured.
#
# (5). If a protocol match is found, needs to check action on access list.
#      a. If it is permit, then the acl_block would not be effected.
#      b. If it is deny, then the acl_block would be effected.
# 
# (6). If there is no address match, go to check next global configuration
#      and put it as implicit deny. If there is an address match, do not need
#      to go to next global configuration.
# 
# Please look at CCNA 1 and 2 book p.846 as reference
#
#----------------------------------------------------------------

global acl_block
global found_d_addr_hello
global found_d_addr_intf

##############################################################################
#
#  Description:   get_subnet
#                 Get subnet address by given ip address and its subnet mask
#  IN:            ip_address: The ip address where you want to know its subnet
#                 subnet_mask: Which you need to use for telling the subnet bits.
#  OUT:           return subnet address
#
##############################################################################

proc get_subnet { ip_address subnet_mask } {

    global verbose

    regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} \
            $ip_address dummy ip_addr1 ip_addr2 ip_addr3 ip_addr4
    regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} \
            $subnet_mask dummy subnet_mask1 subnet_mask2 \
            subnet_mask3 subnet_mask4

    # Logically And between ip_address and subnet_mask

    set ip_subnet1 [expr {$ip_addr1 & $subnet_mask1}]
    set ip_subnet2 [expr {$ip_addr2 & $subnet_mask2}]
    set ip_subnet3 [expr {$ip_addr3 & $subnet_mask3}]
    set ip_subnet4 [expr {$ip_addr4 & $subnet_mask4}]

    return $ip_subnet1.$ip_subnet2.$ip_subnet3.$ip_subnet4
}

##############################################################################
#
#  Description:   if_same_subnet
#                 Check if the interface address and the access list address
#                 are in same subnet
#  IN:            intf_addr interface addres
#                 intf_mask interface subnet musk
#                 access list ip address
#  OUT:           return 1 if they are in same subnet
#                 return 0 if they are not in same subnet
#
##############################################################################

proc if_same_subnet { intf_addr intf_mask acl_address } {

    global verbose

    set intf_subnet [ get_subnet $intf_addr $intf_mask ]
    set acl_subnet [ get_subnet $acl_address $intf_mask ]

    if { $intf_subnet == $acl_subnet } {
        return 1
    } else {
        return 0
    }

}

##############################################################################
#
#  Description:   if_addr_in_acl
#                 Check if the access list is enabled on the associated 
#                 ip_address
#
#  IN:            check_addr: which we want to check if it is in the acl 
#                             config.
#                 acl_addr:   acl address in acl configuration
#                 acl_wildcard: acl wildcard in acl configuration
#
#  OUT:           return 1 if it is effected
#                 return 0 if it is not effected
#
##############################################################################

proc if_addr_in_acl { check_addr acl_addr acl_wildcard } {

    global verbose

    regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} \
            $check_addr dummy check_a(1) check_a(2) \
            check_a(3) check_a(4)


    regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} \
            $acl_addr dummy acl_a(1) acl_a(2) acl_a(3) acl_a(4)


    if { $verbose == 1 } {
        puts "acl_addr is $acl_addr, acl_a(1) is $acl_a(1), acl_a(2)\
                is $acl_a(2),  acl_a(3) is $acl_a(3), acl_a(4) is $acl_a(4)"
    }

    regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} \
            $acl_wildcard dummy acl_w(1) acl_w(2) acl_w(3) acl_w(4)

    set count 1

    while {$count < 5} {
        if { $acl_w($count) == 255 } {

            # If wildcard is 0, the ip address check 
            # should be ignored, keep it effected
            set match_acl($count) 1
        } elseif { $acl_w($count) == 0 } {
            if { $check_a($count) == $acl_a($count) } {
                set match_acl($count) 1
            } else {
                set match_acl($count) 0
            }
        } else {
            # The wildcard takes effection partially
            if { $check_a($count) == $acl_a($count) } {
                set match_acl($count) 1
            } else {                 
                set need_match_ip [expr { $acl_a($count) & \
                    $acl_w($count) }]
		
                if { $need_match_ip == $check_a($count)} {
                    # Keep it as effected
                    set match_acl($count) 1
                } else {
                    set match_acl($count) 0
                }
            }
        } 
        set count [expr $count + 1]
    }

    set check_in_acl 1
    set count 1

    while {$count < 5} {
        if { $match_acl($count) == 0 } {
            # As soon as there is one bit does not match 
            # There is no need to check others. 
            set check_in_acl 0
            return $check_in_acl
        }
        set count [expr $count + 1]
    }

    if { $verbose == 1 } {
        puts "check_in_acl is $check_in_acl"
    }

    return $check_in_acl
}

##############################################################################
#
#  Description:   get_acl_msg
#  IN:            intf: interface name
#                 acl_no: acl number
#                 acl_config: acl configuration in show run
#  OUT:           msg for access list blocking.
#
##############################################################################

proc get_acl_msg { acl_block c_exec intf acl_no acl_config } {

    global verbose

    if { $acl_block == 0 } {
        append msg " The problem does not look like acl problem, but manual " 
        append msg "check $intf in access group $acl_no would be safe. "
        if {$acl_config != "" } {
            append msg " with configuration \"$acl_config\" to be safe"
        }
    } elseif { $acl_block == 1 } {
        set msg "access list problem. you have $intf in access group "
        append msg "$acl_no"
        if { $acl_config != "" } {
            append msg " with configuration \"$acl_config\" "
            if { $c_exec == 0 } {
                append msg " Please check and if so, remove it"
            } else {
                append msg " and it is implicitly denying the OSPF neighbor "
                append msg "ip address. Please check and if so, permit the "
                append msg "OSPF neighbor address."
            }
        }
    } elseif { $acl_block == 2 } {
        set msg "access list configured on near end. you have $intf in access group "
        append msg "$acl_no"
        if { $acl_config != "" } {
            append msg " with configuration \"$acl_config\"."
        }
    }
    return $msg
}

##############################################################################
#
#  Description:   get_acl_config
#  IN:            acl_config: original one without adding the new acl_line
#                  acl_line: New configuration processing line
#  OUT:           acl_msg after the new processes line
#
##############################################################################

proc get_acl_config { acl_config acl_line } {

    global verbose

    if { $acl_config == "" } {
        set acl_config $acl_line
    } else {
        append acl_config " / $acl_line"
    }

    return $acl_config
}


##############################################################################
#
#  Description:   check_s_addr
#
#  IN:            (1). interface address, subnetmask
#                 source addresses
#                 (2). host or subnet
#                 (3). host address or wildcard
#                 
#  OUT:           0: found the address unmatch
#                 1: found the address match
#                 2: found the address match likely
#
##############################################################################

proc check_s_addr { intf_addr subnet_mask addr1 addr2 } {
    
    global verbose

    set found_s_addr 0

    if { $addr1 == "host" } {
        # For a host case, source address has to be remote host

        # it can not be the near end itself
        if { $intf_addr != $addr2 } {
            set found_s_addr [if_same_subnet $intf_addr $subnet_mask $addr2]
        }
        
        # If it is a near end 
        # keep found_s_addr as 0. 

    } elseif [ regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} $addr1 ] {
        # For a subnet case with wildcard, check if remote host is in the
        # subnet, since remote host address is unknown and we know it is
        # in the same subnet of near end, we use near end interface
        # address
        if [ regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} $addr2 ] {
            set found_s_addr [if_addr_in_acl $intf_addr $addr1 $addr2 ]
            if { $verbose == 1 } {
                puts "found_s_addr is $found_s_addr"
            }
        } else {
            puts "This is a format we do not support"
        }
    } else {
        puts "This is a format we do not support"
    }

    return $found_s_addr

}

##############################################################################
#
#  Description:   check_proto_action
#  IN:            acl number
#                 acl protocol
#                 acl action
#                 interface address
#                 found_s_addr_match if it is likely, try to ping 
#                 addr2 for source address
#                 
#  OUT:           0: break instead of continuous execution
#                 1: continuous execution
#
##############################################################################

proc check_proto_action { acl_no acl_proto acl_action intf_addr \
        found_s_addr_match addr2 found_d_addr_match d_addr} {

    global acl_block
    global verbose
    global found_d_addr_hello
    global found_d_addr_intf

    set c_exec 1

    if { $verbose == 1 } {
        puts "acl_proto is $acl_proto, acl_action is $acl_action end"
    }
 
    if { [ regexp {o.*} $acl_proto ] || [ regexp {i.*} $acl_proto ] } {
                
        if { $acl_action == "permit" } {

            # If ospf neighbor is already up 
            # access-list 100 permit ip host 131.107.1.2 host 224.0.0.5
            # access-list 100 permit ospf host 131.107.1.2 host 224.0.0.5
            # will continuously keep up

            # If ospf neighbor is down
            # For example at the following configuration:
            # access-list 100 deny tcp any any
            # access-list 100 permit ip host 131.107.1.2 host 224.0.0.5
            # access-list 100 permit ospf host 131.107.1.2 host 224.0.0.5
            # near end will stuck at EXSTART since near end is waiting for
            # remote end ospf/ip message to start negotiation. At this case, 
            # near end needs to have permit for a packet from remote end
            # destained to itself.
            # For example:
            # access-list 100 permit ospf host 131.107.1.2 host 131.107.1.1
            # access-list 100 permit ip host 131.107.1.2 host 131.107.1.1

            # if found_s_addr_match == 1, keep acl_block at it is
            if { $found_s_addr_match == 2 } {
                # Not sure if it is the access list problem
                # because it can be the case for 
                # access-list 100 permit ip host 131.107.1.3 host 224.0.0.5
                set acl_block 2
            }   

            # Check ping for standard access list
            # If it is able to ping remote side at ip protocol
            # then it should not be the acl problem
            # For extended acl, ping should not be checked at all the cases.
            # For example:
            # (1). For permitted ospf protocol case, ping should not checked
            # (2). For a destination to itself interface only, ping should not
            #      be checked.since it might not allow hello multicast address 
            #      as destination address
            # The following configuration:
            # access-list 100 permit ip any host 131.107.1.1
            # without the configuration presenting:
            # access-list 100 permit ip host 131.107.1.3 host 224.0.0.5 
            # should not be checked.

            if { $acl_no <= 99 } {
                if [regexp {i.*} $acl_proto] {
                    # check ping for ip protocol permit
                    set ping_result [ping_command $intf_addr]

                    if { $ping_result > 60 } {
                        if { $found_s_addr_match == 2 } {
                            set ping_result [ping_command $addr2]
                            if { $ping_result > 60 } {
                                if { $verbose == 1 } {
                                    puts "Ping ok. not like acl problem"
                                }
                                set acl_block 0
                                set c_exec 0
                            } else {
                                set acl_block 2
                            }
                        }
                    }
                }
            } elseif { ( $acl_no > 99 ) &&  ( $acl_no < 199 ) } {

                if { $found_d_addr_match == 1 } {
                    # It means it is a match on "224.0.0.5"
                    if { $found_d_addr_intf == 1} {
                        if { $verbose == 1 } {
                            puts "both destinations are allowed, not like the \
                                    problem"
                        }
                        set acl_block 0
                        set c_exec 0
                    } else {
                        set acl_block 1
                        set c_exec 1
                    }
                    # For an else case, keep looking and keep acl_block and
                    # c_exec condition
                        
                } elseif { $found_d_addr_match == 2 } {
                    # It means it is an interface host match
                    # for destination address
                    if { $found_d_addr_hello == 1} {
                        if { $verbose == 1 } {
                            puts "both destinations are allowed, not like the \
                                    problem"
                        }
                        set acl_block 0
                        set c_exec 0
                    } else {
                        set acl_block 1
                        set c_exec 1
                    }
                }

                if { $verbose == 1 } {
                    puts "check_proto_action: acl_block is $acl_block"
                }
                if  [regexp {i.*} $acl_proto] {
                    # check ping for ip protocol permit
                    set ping_result [ping_command $intf_addr]

                    if { $ping_result > 60 } {
                        if { ($found_s_addr_match == 2) } {
                            set ping_result [ping_command $addr2]
                            if { $ping_result > 60 } {
                                if { $verbose == 1 } {
                                    puts "Ping ok. not like acl problem"
                                }
                                set acl_block 0
                                set c_exec 0
                            } else {
                                set acl_block 2
                            }
                        }
                    }
                }
            }
        } elseif [regexp "deny" $acl_action] {
            # We need to exit the look to not later one
            # because if there is an address match the later one would
            # not take effect. This applies to both standard acl and 
            # extended acl.
            # For example: 
            # access-list 1 deny   131.107.0.0 0.0.255.255
            # access-list 100 deny   ospf any any

            set acl_block 1
            set c_exec 0
        }
    }  else {
        # The protocol does not match, keep c_exec as 1
        set acl_block 1
    }

    if { $verbose == 1 } {
        puts "check_proto_action acl_block is $acl_block"
    }

    return $c_exec 
}


##############################################################################
#
#  Description:   check_std_acl
#                 standard access list processes the following 4 address 
#                 formats and 2 actions:
#                 any, host, host with wildcard
#                 (1). Address is any
#                 case 1: access-list 1 deny any
#                 case 2: access-list 1 permit any
#
#                 (2). Address with host keyword
#                 Example of cli:
#                 case 3: access-list 1 deny host 131.107.1.2
#                 case 4: access-list 1 permit host 131.107.1.2
#
#                 (3). Address without host keyword and without wildcard
#                      No wildcard presenting means wildcard is 0.
#                      The ip address is a host
#                 Example of cli:
#                 case 5: access-list 1 deny 131.107.1.0
#                 case 6: access-list 1 deny 131.107.1.2
#                 case 7: access-list 1 permit 131.107.1.2
#
#                 (4). Address is subnet case with wildcard
#                 case 8: access-list 1 deny 131.107.1.0 255.255.255.0 
#                 
#  IN:            one line acl configuration
#  OUT:           if still need to look at next acl line
#
##############################################################################

proc check_std_acl { acl_no acl_line intf_addr subnet_mask} {

    global acl_block
    global verbose
    set found_addr_match 0

    set c_exec 1

    if [ regexp "access-list $acl_no (.*)" $acl_line dummy acl_arg ] {

        regsub -all "  *" $acl_arg { } acl_arg

        # Need to use non space regexp in order to get acl_action only, not with
        # other any later argument.

        if [regexp {([^ ]*) (.*)} $acl_arg dummy acl_action acl_addr] {

            set addr_length [llength [split $acl_addr " "]]

            if { $verbose == 1 } {
                puts "check_std_acl: addr_length is $addr_length,\
                        acl_action is $acl_action, acl_addr is $acl_addr"
            }

            if { $addr_length == 1 } {
                #  Check if the address is any
                if { $acl_addr == "any" } {

                    set found_addr_match 1
                    set acl_addr ""

                } elseif [regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} $acl_addr] {
                    set found_addr_match [check_s_addr $intf_addr $subnet_mask \
                        "host" $acl_addr]
                } else {
                    puts "It is a case we do not handle"
                }
            } elseif { $addr_length == 2 } {

                set addr1 [lindex $acl_addr 0]
                set addr2 [lindex $acl_addr 1]

                set found_addr_match [check_s_addr $intf_addr $subnet_mask \
                        $addr1 $addr2]

                if { $verbose == 1 } {
                    puts "found_addr_match is $found_addr_match"
                }
            }
        } else {
            puts "Unsupported format"
        }

        if { ($found_addr_match == 1) || ($found_addr_match == 2) } {
            # All standard acls should have ip as their protocol
            set c_exec [check_proto_action $acl_no "ip" $acl_action $intf_addr \
                    $found_addr_match $acl_addr "" ""]
        
        } else {
            set acl_block 1
        }
    } else {
        puts "Unsupported format"
    }
    
    if { $verbose == 1 } {
        puts "check_std_acl: c_exec is $c_exec"
    }

    return $c_exec
}

##############################################################################
#
#  Description:   check_eacl_addr_length_2
#
#                 access-list 100 permit tcp any any 
#                 access-list 100 permit udp any any
#                 access-list 100 deny ospf any any
#                 access-list 100 permit ospf any any
#                 would allow ospf packets go through
#                 Check if the protocol is ospf
#                 if first letter is o for ospf
#
#  IN:            accesses: acl addresses in acl configuration
#                 acl_action: permit or deny
#                 acl_proto: acl protocol 
#                 intf_addr: interface address
#                 subnet_mask: interface subnet mask
#                 
#  OUT:           if still need to look at next acl line
#
##############################################################################

proc check_eacl_addr_length_2 {acl_no addresses acl_action acl_proto \
        intf_addr subnet_mask} {

    global acl_block
    global verbose

    set s_addr [lindex $addresses 0]
    set d_addr [lindex $addresses 1]

    set c_exec 1

    if { $verbose == 1 } {
        puts "s_addr is $s_addr, d_addr is $d_addr"
    }

    if { ($s_addr == "any") && ( $d_addr == "any" ) } {

        set found_addr_match 1
        set c_exec [check_proto_action $acl_no $acl_proto $acl_action \
                $intf_addr $found_addr_match "" "" ""]

    } else {
        puts "the cli is not supported"
    }

    return $c_exec
}

##############################################################################
#
#  Description:   check_d_addr
#
#  IN:            destination addresses
#                 (1). host or subnet
#                 (2). host address or wildcard
#                 
#  OUT:           0: found the address match
#                 1: found the address unmatch
#                 2: found the address match likely
#
##############################################################################

proc check_d_addr { intf addr3 addr4 } {

    global verbose
    global acl_block
    global found_d_addr_hello
    global found_d_addr_intf

    set found_d_addr 0

    if { $addr3 == "host" } {
        # For a host case
        if { $addr4 == "224.0.0.5" } {
            set found_d_addr_hello 1
            set found_d_addr 1
        } else {
            if { $addr4 == $intf } {
                set found_d_addr_intf 1
                set found_d_addr 2
            }
            # keep found_d_addr as 0
            set acl_block 1
        }
    } elseif [ regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} $addr3 ] {
        # For a subnet case with wildcard

        set found_d_addr_hello_new [if_addr_in_acl "224.0.0.5" $addr3 $addr4 ]

        if { $found_d_addr_hello != $found_d_addr_hello_new } {
            set found_d_addr_hello $found_d_addr_hello_new
        }

        set found_d_addr_intf_new [if_addr_in_acl $intf $addr3 $addr4 ]

        if { $found_d_addr_intf != $found_d_addr_intf_new } {
            set found_d_addr_intf $found_d_addr_intf_new
        }

        if { $found_d_addr_intf_new == 1 } {
            set found_d_addr 2
        }
        
        if { $verbose == 1 } {
            puts "found_d_addr is $found_d_addr"
        }
    }  
    return $found_d_addr
}


##############################################################################
#
#  Description:   check_eacl_addr_length_3
#
#                 For the following 2 categories:
#                 (1). Source address is a host address
#                     access-list 100 permit ip host 131.107.1.1 any
#                 (2). Destination address is a host address
#                     access-list 100 permit ip any host 131.107.1.1
#                 Ospf is sending hello packet to its destination by 
#                 multicast address 224.0.0.5 from its neighbor remote
#                 host, so we need to have configuration to permit them
# 
#                 For seven keyword configurations, the following would be the
#                 minimus configuration:
#                 access-list 101 permit ip any host 224.0.0.5
#                 or
#                 access-list 100 permit ip host 131.107.1.2 any             
#
#  IN:            addresses including host and destination address
#                 interface address
#                 interface subnet mask
#                 
#  OUT:           0: there is no need to look at next acl line
#                 1: still need to look at next acl line
#
##############################################################################

proc check_eacl_addr_length_3 { acl_no addresses acl_action acl_proto \
        intf_addr subnet_mask } {

    global acl_block
    global verbose
    set found_addr_match 0

    set c_exec 1
          
    set addr1 [lindex $addresses 0]
    set addr2 [lindex $addresses 1]
    set addr3 [lindex $addresses 2]

    if { $addr1 == "any" } {
        # means source address is always match, needs to check destination
        # address
        set found_s_addr_match 1
        set s_addr ""
        set found_d_addr_match [check_d_addr $intf_addr $addr2 $addr3]
        set found_addr_match $found_d_addr_match

        if { $verbose == 1 } {
            puts "found_d_addr_match is $found_d_addr_match"
        }
        set d_addr $addr3
    } elseif { $addr3 == "any" } {

        set found_d_addr_match 3
        set d_addr ""
        # mean destination address is always match, needs to check 
        # source address

        set found_s_addr_match [check_s_addr $intf_addr $subnet_mask \
                $addr1 $addr2]
        set s_addr $addr2
        if { $verbose == 1 } {
            puts "found_s_addr_match is $found_s_addr_match"
        }
        set found_addr_match $found_s_addr_match
        set d_addr ""

    } else {
        puts "unsupported format"
        return
    }

    if { $verbose == 1 } {
        puts "found_addr_match is $found_addr_match"
    }
    
    if { ($found_addr_match == 1) || ($found_addr_match == 2) } {
        # check if protocol and action match
        
        set c_exec [check_proto_action $acl_no $acl_proto $acl_action \
                $intf_addr $found_s_addr_match $s_addr $found_d_addr_match \
                $d_addr]
    }

    return $c_exec
}

##############################################################################
#
#  Description:   check_eacl_addr_length_4
#
#                 surpporting the following 4 categories:
#                 (1). Both are host
#                     access-list 100 permit ip host 131.107.1.1 host \
#                     131.107.1.2
#                 (2). Source is host and destination is subnet
#                     access-list 100 permit ip host 131.107.1.1 \
#                     131.107.1.0 0.0.0.255
#                 (3). Source is subnet and destination is host
#                     access-list 100 permit ip 131.107.1.0 0.0.0.255 \
#                     host 131.107.1.2
#                 (4). Both are subnet
#                     access-list 100 permit ip 131.107.1.0 0.0.0.255 \
#                     224.0.0.0 0.0.0.255
#                  
#                 multicast address 224.0.0.5 from its neighbor remote
#                 host, so we need to have configuration to permit them
# 
#                 For eight keyword configurations, the following would be the
#                 minimum configuration:
#                 access-list 100 permit ip host 131.107.1.2 host 224.0.0.5
#
#  IN:            addresses including host and destination address
#                 interface address
#                 interface subnet mask
#                 
#  OUT:           0: there is no need to look at next acl line
#                 1: still need to look at next acl line
#
##############################################################################

proc check_eacl_addr_length_4 { acl_no addresses acl_action acl_proto \
        intf_addr subnet_mask } {

    global acl_block
    global verbose

    set found_d_addr_match 0
    set found_s_addr_match 0

    set c_exec 1
                
    set addr1 [lindex $addresses 0]
    set addr2 [lindex $addresses 1]
    set addr3 [lindex $addresses 2]
    set addr4 [lindex $addresses 3]

    # For all supported formats, addr2 and addr4 should be 4 octet
    # ip address or wildcard format.

    if { ![ regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} $addr2 ] || \
            ![ regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} $addr4 ] } {
        puts "unsupported format"
        return $c_exec
    }

    set found_d_addr_match [check_d_addr $intf_addr $addr3 $addr4]

    if { $found_d_addr_match == 1 } {
        set found_s_addr_match [check_s_addr $intf_addr $subnet_mask \
                $addr1 $addr2]
    }

    if { ($found_s_addr_match == 1) || ($found_s_addr_match == 2) } {
        set c_exec [check_proto_action $acl_no $acl_proto $acl_action \
                $intf_addr $found_s_addr_match $addr2 $found_d_addr_match $addr4]
    } else {
        set acl_block 1
    }

    return $c_exec
}

##############################################################################
#
#  Description:   check_extended_acl
#                 The difference between standard access list and extended 
#                 access list is standard access list only needs to check 
#                 source address when it permits any destination address
#                 extendend access list needs to check both source and 
#                 destination address
#
#                 extended access list has the following format as its cli:
#                 (1). Address is any
#                 case 9:  access-list 100 permit tcp any any 
#                 case 10: access-list 100 permit udp any any
#                 permit tcp any means permit tcp only, no ip is permitted.
#
#                 (2). Address with host keyword
#                 source address as host and destination end is any
#                 case 11: access-list 100 permit tcp host 131.107.1.1 any
#
#                 source address as any and destination as host
#                 case 12: access-list 100 permit tcp any host 131.107.1.1
#
#                 both end as hosts
#                 case 13:  access-list 100 permit tcp host 131.107.1.1 \
#                           host 131.107.1.2
#
#                 access-list 100 permit tcp 131.107.1.0 0.0.0.255 any
#                    (source address as subnet with wildcard)
#                 access-list 100 permit tcp 131.107.1.0 0.0.0.255 \
#                               131.107.1.0 0.0.0.255
#                    (both ends are subnet with wildcard)
#
#  IN:            access list number 
#                 one line acl configuration
#                 interface address
#                 interface subnet mask
#                 
#  OUT:           if still need to look at next acl line
#
##############################################################################

proc check_extended_acl { acl_no acl_line intf_addr subnet_mask} {

    global acl_block
    global found_addr_match_likely
    global verbose

    set c_exec 1

    if [ regexp "access-list $acl_no (.*)" $acl_line dummy acl_arg ] {
        regsub -all "  *" $acl_arg { } acl_arg

        # Need to use non-space regexp in order to get acl_action only, not with
        # other any later argument

        if [regexp {([^ ]*) ([^ ]*) (.*)} $acl_arg dummy \
                acl_action acl_proto addresses] {

            set addr_length [llength [split $addresses " "]]

            if { $verbose == 1 } {
                puts "acl_arg is $acl_arg, acl_action is $acl_action,\
                      acl_proto is $acl_proto,  addresses is $addresses, \
                      addr_length is $addr_length"
            }
                
            if { $addr_length == 2 } {

                set c_exec [check_eacl_addr_length_2 $acl_no $addresses \
                        $acl_action $acl_proto $intf_addr $subnet_mask]

            } elseif { $addr_length == 3 } {

                set c_exec [check_eacl_addr_length_3 $acl_no $addresses \
                        $acl_action $acl_proto $intf_addr $subnet_mask]

            } elseif { $addr_length == 4 } {

                set c_exec [check_eacl_addr_length_4 $acl_no $addresses \
                        $acl_action $acl_proto $intf_addr $subnet_mask]

            }
        }
    }

    return $c_exec

}


##############################################################################
#
#  Description:   check_acl_problem
#
#  IN:            interface name
#                 intf_addr
#                 remote message
#                 existing message
#                 
#  OUT:           final message
#
#############################################################################

proc check_acl_problem { intf r_msg msg } {

    global acl_block
    global found_d_addr_hello
    global found_d_addr_intf
    global verbose

    set ios_cli "show run int $intf | i ip address"
    set ip_addr_out [ios_enable_command $ios_cli]

    if [regexp {ip address (.*) (.*)$} $ip_addr_out dummy \
            intf_addr subnet_mask] {

        if { $verbose == 1} {
            puts "ip address is $intf_addr \
                    subnet_address is $subnet_mask"
        }
    }

    #(1). Check if the interface has an acl configured

    set ios_cli "show run int $intf | include ip access-group"
    set cli_out [ios_enable_command $ios_cli]

    #(2). If there is an acl configured on the interface, check the acl
    # number.
    # It might have 2 config lines for one interface, one for in and the other
    # for out. For example:
    # interface GigabitEthernet0/0
    #  ip access-group 1 in
    #  ip access-group 1 out
    # out is not effected


    set found_d_addr_hello 0
    set found_d_addr_intf 0

    if [ regexp {ip access-group ([0-9]+) in} $cli_out dummy acl_no] {

        # The minimum required config for ospf is:
        # source address: remote host interface
        # destination address: multicast ip address 224.0.0.5
        # protocol: ip or ospf
        # action: permit
        # For standard acl, there is no destination and protocol check 
        # since it does not have it
        # access-list 1 permit 131.107.1.2
        # For extended acl, the following is minimum:
        # access-list 101 permit ip host 131.107.1.2 host 224.0.0.5

        set acl_block 0
        set acl_config ""

        #(3). Check if there is a global configuration for the access list.
        set ios_cli "show run | i access-list $acl_no"
        set acl_out [ios_enable_command $ios_cli]

        if [ regexp "access-list $acl_no" $acl_out ] {

            # If there is an access list found in global configuration, 
            # needs to check if it is the reason to block.

            foreach acl_line [split $acl_out "\r\n"] {

                if [ regexp "access-list $acl_no " $acl_line ] {

                    set acl_config \
                            [get_acl_config $acl_config $acl_line ]

                    if { $verbose == 1 } {
                        puts "acl_config is $acl_config"
                    }

                    #a. check if it is a standard access list 
                    if { $acl_no <= 99 } {

                        set c_exec [check_std_acl \
                                $acl_no $acl_line $intf_addr $subnet_mask]
                    
                        if { $verbose == 1 } {
                            puts "acl_block is $acl_block c_exec is $c_exec"
                        }

                        if { $c_exec == 0 } {
                            break
                        }

                    } elseif { ($acl_no > 99) && ($acl_no <= 199) } {

                        set c_exec [check_extended_acl \
                                $acl_no $acl_line $intf_addr $subnet_mask]

                        if { $c_exec == 0 } {
                            break
                        }

                        if { $verbose == 1 } {
                            puts "acl_block is $acl_block"
                        }

                    } elseif { $acl_no > 199 } {
                        append msg $r_msg
                        append msg "or addess list problem. The diagnostic "
                        append msg "script does not support access list "
                        append msg "other than standard acl and extended acl "
                        append msg "need manually check it" 
                    }
                }
            }
        }

        #(4) check access list block result

        if { $acl_config != "" } {
            if { $acl_block == 0 } {
                append msg $r_msg
                set acl_msg [get_acl_msg $acl_block $c_exec $intf $acl_no \
                        $acl_config]
                append msg $acl_msg

            } elseif { $acl_block == 1 } {
        
                set acl_msg [get_acl_msg $acl_block $c_exec $intf $acl_no \
                        $acl_config]
                append msg $acl_msg

            } elseif { $acl_block == 2 } {

                # acl_block 2 means we are not sure what the reason is, so 
                # list all of them 
                append msg $r_msg
                set acl_msg [get_acl_msg $acl_block $c_exec $intf $acl_no \
                        $acl_config]
                append msg $acl_msg 
        
            }
        }
    } else {
        append msg $r_msg
    }
}

