::cisco::eem::event_register_syslog occurs 1 pattern "\%OSPF-5-ADJCHG:.*DOWN, Neighbor Down: Interface down or detached"

#------------------------------------------------------------------
# EEM policy to report problem on ospf neighbor down is caused by
# (1). The interface shut in its own router is configured (admin down)
# (2). An interface shut in remote end is configured and near end has a 
#      Down Down
# (3). Passive-interface is configured.
#      config a passive interface in router ospf protocol.
#      c7200-6(config-if)#router ospf 107
#      c7200-6(config-router)#passive-interface giga0/0
#
# (4). The ospf configuration is removed
#      For example:
#      you have the following configuration in ios
#      router ospf 107 
#        log-adjacency-changes
#        network 131.107.0.0 0.0.255.255 area 0
#        neighbor 131.107.5.11
#      remove it by "no router ospf 107"
#
# May 2007 - Christine Yan (cyan@cisco.com)
#
# Copyright (c) 2007 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-05/15/2007
#
# Name: ospf_intf_down_detached.tcl
#
# Desc: This script is an application to diagnose ospf problem. It reports
#       ospf problem to customer the possible reasons as "shut", "down down"
#       or passive interface.
#
# Environment Variables for this script are:
#  _send_email_for_intf_down_detached (opt)
#  _svcdiag_intfConfigHistory (opt)
#  _svcdiag_intfEventHistory  (opt)
#  _email_server (opt)
#  _email_from  (opt)
#  _email_to    (opt)
#  _email_cc    (opt)
#
# IOS config:
#
#  conf t
#
#  event manager environment _send_email_for_intf_down_detached no
#  event manager environment _svcdiag_intfConfigHistory FALSE
#  event manager environment _svcdiag_intfEventHistory FALSE
#  event manager directory user policy disk0:/svc_diag
#  event manager directory user library disk0:/user_lib
#  event manager policy ospf_intf_down_detached.tcl type user
#
#  event manager environment _email_server email.cisco.com
#  event manager environment _email_from cyan@cisco.com
#  event manager environment _email_to cyan@cisco.com
#  event manager environment _email_cc cyan@cisco.com
#
#  Please note: on some devices you do have to use
#  event manager directory user policy disk2:/
#
#  For 3825 or 3xxx serial of routers, should use flash: instead of disk0:
#
# How to run this script:
#
# As soon as the script is registered in config mode, it can be run since it 
# accepts a syslog event. 
# 
# so run the script by registering it:
# c7200-6(config)#event manamger policy ospf_intf_down_detached.tcl
#
# where ospf_intf_down_detached.tcl is the name of this script
#
#------------------------------------------------------------------
# The namespace import commands that follow import some cisco TCL extensions
# that are required to run this script

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

# User will make decision on if the email will be sent to them. This is 
# by environment variable _send_email_for_intf_down_detached
#
set send_email 0

if {[info exists _send_email_for_intf_down_detached]} {

    if [regexp yes $_send_email_for_intf_down_detached] {

	# Check if all the env variables we need exist
	# If any of them doesn't exist, print out an error msg and quit

	if {![info exists _email_server]} {
	    set result \
		"Policy cannot be run: variable _email_server has not been set"
	    error $result "_email_server is required for sending email"
	}
	if {![info exists _email_from]} {
	    set result \
		 "Policy cannot be run: variable _email_from has not been set"
	    error $result "_email_from is required for sending email"
	}
	if {![info exists _email_to]} {
	    set result \
		    "Policy cannot be run: variable _email_to has not been set"
	    error $result "_email_to is required for sending email"
	}
	if {![info exists _email_cc]} {
	    #_email_cc is an option, must set to empty string if not set.
	    set _email_cc ""
	}

	set send_email 1	
    }
}     

global verbose
set verbose 0

if {[info exists _verbose_for_intf_down_detached]} {

    if [regexp yes $_verbose_for_intf_down_detached] {

	set verbose 1

    }
}

set config_history 0

if {[info exists _svcdiag_intfConfigHistory]} {
    if { $_svcdiag_intfConfigHistory == "TRUE" } {
	set config_history 1
    }
}

set event_history 0

if {[info exists _svcdiag_intfEventHistory]} {
    if { $_svcdiag_intfEventHistory == "TRUE" } {
	set event_history 1
    }
}

array set arr_einfo [event_reqinfo]
set syslog_msg $arr_einfo(msg)

# Get neighbor ip address and the interface name from syslog message.

if [regexp {Nbr (.*) on (.*) from} $syslog_msg dummy neigh_addr intf] {
    if { $verbose == 1 } {
        puts "Nbr is $neigh_addr, intf is $intf"
    }
} else {
    exit
}

# Check if the problem is caused by interface shut at its own interface
set cli_cmd "show ip int brief | include $intf"
set cli_out [ios_enable_command $cli_cmd]

set interface_problem 0

set msg_header "\n\nTHE SERVICE DIAG MESSAGE FOR INTERFACE DOWN DETACHED is:"
set msg $msg_header

if [regexp {administratively down} $cli_out dummy] {
    
    append msg \n\n "OSPF neighbor $neigh_addr down is caused by interface "
    append msg "shut down. Please \"no shut\" the interface $intf to bring "
    append msg "the adjacency up"

} elseif [regexp {down                  down} $cli_out dummy] {
    set interface_problem 1
} elseif [regexp {up                    down} $cli_out dummy] {
    set interface_problem 1
} else {

    set ios_cli "show run int $intf | include ip address"

    set cli_out [ios_enable_command $ios_cli]
    if {$verbose == 1} {
	puts "\n cli_out=$cli_out"
    }
    # Now check if the error message is caused by passive-interface
    # configuration

    # Some user can configure a non ip address there, this case the
    # passive interface would not take effection.
    # For example, at the following configuration:
    #
    # interface FastEthernet2/0
    # no ip address     <-----
    # shutdown
    # duplex auto
    # speed auto
    # end

    if [ regexp "no ip address" $cli_out ] {

        append msg "The interface $intf is not in ospf network, needs to "
        append msg "have one in order to make it effected"

    } elseif [ regexp {ip address ([0-9]+).([0-9]+).([0-9]+).([0-9]+) .*} \
        $cli_out dummy intf_ip(1) intf_ip(2) intf_ip(3) intf_ip(4) ] {

        # Read the running configuration file to see if "passive-interface" 
        # is configured. 

        # If it is not configured at all, the problem should not be caused by 
        # that

        set ios_cli "show run | include passive-interface $intf"
        set cli_out [ios_enable_command $ios_cli]

        if [regexp {passive-interface} $cli_out dummy] {

            # Verified the following case for passive interface. 
            # The passive interface is configured but it is not effected. 
            #!
            #interface Ethernet0/0
            # ip address 131.107.1.1 255.255.255.0
            #!
            #router ospf 1
            # log-adjacency-changes
            # passive-interface Ethernet0/0  --- This passive interface is not 
            #             valid to cause adj down since it is not in the
            #             network
            # 131.107.1.1 as the ether interface
            # network 1.1.0.0 0.0.255.255 area 1

            if { $verbose == 1 } {
                puts "There is a passive interface configured, need to make " 
                puts "sure it is for this network"
            }

            # Get the ospf number for this network
            set ios_cli "show run"
            set cli_out [ios_enable_command $ios_cli]

            # We might get a passive-interface configuration for this interface
            # but the interface is not in same subnet as the configured network
            # in the ospf 

            set found_passive_config 0

            foreach line [split $cli_out "\r\n"] {

                if { $found_passive_config == "1" } {
    
                    # We need to pay attention here.
                    # Read next line to see if the interface is in the 
                    # ospf network

                    if [regexp {network ([0-9]+).([0-9]+).([0-9]+).([0-9]+) ([0-9]+).([0-9]+).([0-9]+).([0-9]+) area .*} $line dummy ip(1) ip(2) ip(3) ip(4) s_wildcard(1) s_wildcard(2) s_wildcard(3) s_wildcard(4)] {

                        if { $verbose == 1 } {
                            puts "ip is $ip(1).$ip(2).$ip(3).$ip(4)"
                            puts "wildcard is $s_wildcard(1).$s_wildcard(2).$s_wildcard(3).$s_wildcard(4)"
                        }

                        set count 1
                        while {$count < 5} {
                            if { $s_wildcard($count) == 255 } {
                                # If wildcard is 255, it means it should be 
                                # ignored
                                set effected($count) 1
                            } elseif { $s_wildcard($count) == 0 } {
                                if { $intf_ip($count) == $ip($count) } {
                                    set effected($count) 1
                                } else { 
                                    set effected($count) 0
                                }
                            } else { 
                                # The wildcard takes effection partially
                                if { $intf_ip($count) == $ip($count) } {
                                    set effected($count) 1
                                } else {
                                    set care_value \
                                       [expr {255 - $s_wildcard($count)}]
                                    
                                    set need_match_ip \
                                        [expr { $ip($count) & $care_value }]

                                    if { $need_match_ip == $intf_ip($count)} {
                                        set effected($count) 1
                                    } else {
                                        set effected($count) 0
                                    }
                                }
                            } 
                            set count [expr $count + 1]
                        }

                        set passive_effected 1
                        set count 1
                        while {$count < 5} {
                            if { $effected($count) == 0 } {
                                set passsive_effected 0
                                set found_passive_config 0
                                break
                            }
                            set count [expr $count + 1]
                        }

                        if { $passive_effected == 1 } {
                            break
                        }
                    }
                } elseif { $found_passive_config == "0" } {
                    if [regexp " passive-interface $intf" $line dummy1] {
                        if {$verbose == 1} {
                            puts "dummy1 = $dummy1"
                        }
                        set found_passive_config 1
		    }
		}
	    }

            # For the case $dummy = "" and the passive interface is not
            # effected case:

            if { $passive_effected == 1 } {
                append msg \n\n "OSPF neighbor $neigh_addr is down. Please "
                append msg "check if passive interface is configured on " 
                append msg "$intf and if so, remove it "
            }
        }

        # Now check if the errmsg is caused by mismatched area id
        # Check if the interface is in same subnet of this network
        # configuration. If it is, check its area id.
        # When near end has different area id as the remote end, the
        # interface can be down.

        if { $msg == $msg_header } {

            # Now check if the errmsg is caused by mismatched area id
            # Check if the interface is in same subnet of this network
            # configuration. If it is, check its area id.
            # When near end has different area id as the remote end, the
            # interface can be down.

            set ospf_area_id [get_intf_area_id $intf_ip(1) $intf_ip(2) \
                $intf_ip(3) $intf_ip(4)]
            append msg \n "Nbr $neigh_addr down might be due to "
            if { $ospf_area_id == "" } {
                # Now check if the errmsg is caused by there is no ospf router is
                # configured.
                append msg "ospf process configuration is removed, please "
                append msg "check and if so, add it in"
            } else {
                append msg "unmatched ospf area id, this ospf process "
                append msg "has area id $ospf_area_id. Please check "
                append msg "if remote end has a different ospf area id"
            }
        }
    }
}

if { $interface_problem == "1" } {
    # If the interface is down by layer 1/2 protocol, such as remote 
    # end shut  the same message can happen too.
    append msg \n\n "OSPF neighbor $neigh_addr down might be due to "
    append msg "remote end shut down, which makes ospf neighbor unreachable "
    append msg "or other layer 2 interface protocol problem "
}

#Whatever if there is an email to be sent out for diagnostic message, 
#the final message will be printed out on screen.
if { $msg == $msg_header } {
    append msg \n\n "reason unknown"
}

append msg \n\n "--------------------------------------------------------"
append msg "------"
action_syslog msg $msg

if { $send_email == 0 } {
   send_info $config_history $event_history 
} else {
   send_result_by_email $config_history $event_history $msg $_email_from \
       $_email_to $_email_cc $_email_server
}
