::cisco::eem::event_register_syslog occurs 10 pattern "\%OSPF-4-ERRRCV: Received invalid packet: mismatch area ID"


#------------------------------------------------------------------
# EEM policy to report problem on ospf missing configured area id
#
# May 2007 - Christine Yan (cyan@cisco.com)
#
# Copyright (c) 2007 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-05/15/2007
#
#
# Name: ospf_miss_area_id.tcl
#
# Desc: This script is an application to diagnose ospf problem. It gives 
#       additional information to customer after an missing area id 
#       is configured.
#
# Environment Variables for this script are:
#  _send_email_for_miss_area_id (opt)
#  _svcdiag_missConfigHistory (opt)
#  _svcdiag_missEventHistory  (opt)
#  _email_server (opt)
#  _email_from  (opt)
#  _email_to    (opt)
#  _email_cc    (opt)
#
# IOS config:
#
#  conf t
#
#  event manager environment _send_email_for_miss_area_id no
#  event manager environment _svcdiag_missConfigHistory FALSE
#  event manager environment _svcdiag_missEventHistory FALSE
#  event manager directory user policy disk0:/svc_diag
#  event manager directory user library disk0:/user_lib
#  event manager policy ospf_miss_area_id.tcl type user
#
#  event manager environment _email_server email.cisco.com
#  event manager environment _email_from cyan@cisco.com
#  event manager environment _email_to cyan@cisco.com
#  event manager environment _email_cc cyan@cisco.com
#
#  Please note: on some devices you do have to use
#  event manager directory user policy disk2:/
#
# For 3825 or 3xxx series of routers, should use flash: instead of disk0:
#
#  How to run this script:
#
# As soon as the script is registered in config mode, it can be run after
# a syslog event occures.
# 
# so run the script by registering it:
# c7200-6(config)#event manamger policy ospf_miss_area_id.tcl
#
# where ospf_miss_area_id.tcl is the name of this script
#
#------------------------------------------------------------------
# The namespace import commands that follow import some cisco TCL extensions
# that are required to run this script

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

# check if all the env variables we need exist
# If any of them doesn't exist, print out an error msg and quit

set send_email 0

if {[info exists _send_email_for_miss_area_id]} {

    if [regexp yes $_send_email_for_miss_area_id] {

	# Check if all the env variables we need exist
	# If any of them doesn't exist, print out an error msg and quit

	if {![info exists _email_server]} {
	    set result \
		"Policy cannot be run: variable _email_server has not been set"
	    error $result "_email_server is required for sending email"
	}
	if {![info exists _email_from]} {
	    set result \
		"Policy cannot be run: variable _email_from has not been set"
	    error $result "_email_from is required for sending email"
	}
	if {![info exists _email_to]} {
	    set result \
		    "Policy cannot be run: variable _email_to has not been set"
	    error $result "_email_to is required for sending email"
	}
	if {![info exists _email_cc]} {
	    #_email_cc is an option, must set to empty string if not set.
	    set _email_cc ""
	}

	set send_email 1	
    }
}

global verbose
set verbose 0

if {[info exists _verbose_for_miss_area_id]} {

    if [regexp yes $_verbose_for_miss_area_id] {

	set verbose 1

    }
}

set config_history 0

if {[info exists _svcdiag_missConfigHistory]} {
    if { $_svcdiag_missConfigHistory == "TRUE" } {
	set config_history 1
    }
}

set event_history 0

if {[info exists _svcdiag_missEventHistory]} {
    if { $_svcdiag_missEventHistory == "TRUE" } {
	set event_history 1
    }
}

array set arr_einfo [event_reqinfo]
set syslog_msg $arr_einfo(msg)

# Get the interface name from syslog message. 
if [regexp {virtual-link but not found from .*, (.*)} $syslog_msg dummy intf] {
    if { $verbose == 1 } {
        puts " $syslog_msg is the syslog message we are looking for"
    }
} else {
    exit
}

# Check if the problem is caused by interface shut at its own interface
set cli_cmd "show ip int brief | include $intf"
set cli_out_ip_int_brief [ios_enable_command $cli_cmd]

foreach line [split $cli_out_ip_int_brief "\r\n"] {
    if { [regexp $intf $line dummy ] } {
	set intf_addr [lindex $line 1]
    }
}

regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} \
	$intf_addr dummy intf_addr1 intf_addr2 \
	intf_addr3 intf_addr4

set ospf_area_id [get_intf_area_id $intf_addr1 $intf_addr2 \
	    $intf_addr3 $intf_addr4]

set msg_header "\n\nTHE SERVICE DIAG MESSAGE FOR MISS AREA ID is:"
set msg $msg_header

append msg \n\n "The problem happens when receiving packet(s) from backbone"
append msg "router, which has ospf area id 0, and the ospf network the "
append msg "receiving interface ($intf) participated has "
append msg "a different area id, which is $ospf_area_id, and there is no "
append msg "virtual link attached to this interface. Please reconfigure "
append msg "area id to make them same or configure a virtual link between "
append msg "them"

if { $msg == $msg_header } {
    append msg \n\n "reason unknown"
}

append msg \n\n "--------------------------------------------------------"
append msg "------"

action_syslog priority info msg $msg

if { $send_email == 0 } {
   send_info $config_history $event_history 
} else {
   send_result_by_email $config_history $event_history $msg $_email_from \
       $_email_to $_email_cc $_email_server
}
