::cisco::eem::event_register_timer watchdog name ospf_stuck_timer sec 120 maxrun_sec 1200
#------------------------------------------------------------------
# EEM policy to report problem on ospf stuck during formation. ospf can be
# stuck at different init states: attempt, init, exchange, exstart. 
#
# May 2007 - Christine Yan (cyan@cisco.com)
#
# Copyright (c) 2007 by cisco Systems, Inc.
# All rights reserved.
#
# Version-date: v1.0-05/15/2007
#
# Desc: This script is an application to diagnose ospf problems. It can 
#       Report ospf problem to customer when ospf can not be formed because
#       it is stuck at attempt, init and exstart.
#  (1). Stuck at attempt state can be caused by wrong neighbor configuration
#       as the following configuration:
#       router ospf 107
#       log-adjacency-changes
#        network 131.107.0.0 0.0.255.255 area 0
#        neighbor 131.107.5.11  ----- Neighbor should be 131.107.5.2
#  (2). Stuck at init state can be caused by access list in remote end side.
#       The script can provide suggestion for this case.
#  (3). Stuck at exstart/exchange state can be caused by mtu problem.
# 
#
# Name: ospf_stuck_cases.tcl
#
# Environment Variables for this script are:
#  _send_email_for_stuck_cases  (opt)
#  _svcdiag_stuckConfigHistory (opt)
#  _svcdiag_stuckEventHistory  (opt)
#  _email_server (opt)
#  _email_from  (opt)
#  _email_to    (opt)
#  _email_cc    (opt)
#
# IOS config:
#
#  conf t
#
#  event manager environment _send_email_for_stuck_cases no
#  event manager environment _svcdiag_stuckConfigHistory FALSE
#  event manager environment _svcdiag_stuckEventHistory FALSE
#  event manager directory user policy disk0:/svc_diag
#  event manager directory user library disk0:/user_lib
#  event manager policy ospf_stuck_cases.tcl type user
#
#  event manager environment _email_server email.cisco.com
#  event manager environment _email_from cyan@cisco.com
#  event manager environment _email_to cyan@cisco.com
#  event manager environment _email_cc cyan@cisco.com
#
#  Please note: on some devices you do have to use
#  event manager directory user policy disk2:/
#
#  For 3825 or 3xxx serial of routers, should use flash: instead of disk0:
#
# How to run this script
#
# As soon as the script is registered in config mode, it can be run since it 
# accepts a syslog event. 
# 
# so run the script by registering it:
# c7200-6(config)#event manamger policy ospf_stuck_cases.tcl
#
# where ospf_stuck_cases.tcl is the name of this script
#
#------------------------------------------------------------------
# The namespace import commands that follow import some cisco TCL extensions
# that are required to run this script

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

# Check if all the env variables we need exist
# If any of them doesn't exist, print out an error msg and quit
set send_email 0

if {[info exists _send_email_for_stuck_cases]} {

    if [regexp yes $_send_email_for_stuck_cases] {

	# Check if all the env variables we need exist
	# If any of them doesn't exist, print out an error msg and quit

	if {![info exists _email_server]} {
	    set result \
		    "Policy cannot be run: variable _email_server has not \
		    been set"
	    error $result "_email_server is required for sending email"
	}

	if {![info exists _email_from]} {
	    set result \
		    "Policy cannot be run: variable _email_from has not been \
		    set"
	    error $result "_email_from is required for sending email"
	}

	if {![info exists _email_to]} {
	    set result \
		    "Policy cannot be run: variable _email_to has not been set"
	    error $result "_email_to is required for sending email"
	}

	if {![info exists _email_cc]} {
	    #_email_cc is an option, must set to empty string if not set.
	    set _email_cc ""
	}

	set send_email 1	

    }
}     

global verbose
set verbose 0

if {[info exists _verbose_for_stuck_cases]} {

    if [regexp yes $_verbose_for_stuck_cases] {
	set verbose 1
    }

}

set config_history 0

if {[info exists _svcdiag_stuckConfigHistory]} {
    if { $_svcdiag_stuckConfigHistory == "TRUE" } {
	set config_history 1
    }
}

set event_history 0

if {[info exists _svcdiag_stuckEventHistory]} {
    if { $_svcdiag_stuckEventHistory == "TRUE" } {
	set event_history 1
    }
}

##############################################################################
#
#  Description: check_mtu_problem 
#               mtu problem can cause ospf stuck at 2 places, one is EXTART, 
#               the other is EXCHANGE, which depends on which state the router
#               is at when ospf is starting to have negoaition at hello 
#               protocol. 
#
#  IN: cli out from ios command to see if mtu is configured in the interface 
#      configuration
#  OUT: Return value for diag message
#
##############################################################################

proc check_mtu_problem { state intf } {

    global verbose

    # Read the running configuration file to see if mtu is 
    # configured. 

    # If it is not configured at all, we need to give user suggestion to check
    # remote side. 

    if {![regexp {(.*)\.(.*)} $intf dummy main_intf]} {
	set main_intf $intf
    }
    
    if { $verbose == 1 } {
        puts "main_intf is $main_intf "
    }
    # check mtu for this interface
    set ios_cli "show run int $main_intf | include mtu"
    set cli_out [ios_enable_command $ios_cli]
    if { $verbose == 1 } {
        puts "cli_out is $cli_out "
    }

    set msg_mtu "unmatched mtu, the stuck interface $intf "

    if [regexp {mtu ([0-9]+)} $cli_out dummy mtu_value] {

        append msg_mtu "has mtu value $mtu_value, "
        append msg_mtu "Please check mtu value in remote side to "
        append msg_mtu "make sure they are synchronized"
		
	# When mtu is not synchronized, it is still able to ping since
	# ping packet is using different packet size. 
    } else {

        append msg_mtu "is using default mtu value, "
        append msg_mtu "which is 1500, Please check what mtu value remote "
        append msg_mtu "end is using to make sure they are synchronized"
    }

    if { $verbose == 1 } {
	puts "check_mtu_problem: $msg_mtu"
    }

    return $msg_mtu
}

##############################################################################
#
#  Description: check_neighbor_problem
#               neighbor problem can cause ospf stuck at attempt state when
#               ospf is starting to have negoaition at hello protocol. 
#
#  IN: cli out from ios command to see if mtu is configured in the interface 
#      configuration
#  OUT: Return value for diag message
#
##############################################################################

proc check_neighbor_problem { neigh_addr } {

    # At the stage to check neighbor problem, the cli_out should be only one
    # line

    global verbose

    set msg_neighbor ""

    # Read the running configuration file to see if 
    # "neighbor $neigh_addr" is configured. 

    # If it is not configured at all, the problem should not be 
    # caused by it

    set ios_cli "show run | include neighbor $neigh_addr"
    set cli_out_for_neigh_addr [ios_enable_command $ios_cli]

    # If it is configured, check if the neighbor address is in
    # the interface subnet. If it is same, ping neighbor ip
    # address to see if it is able to ping. 

    # There is no need to check if the neighbor address is in same 
    # ospf network since it has to be when stuck
    # at attempt state due to ospf negotiation.

    if {[regexp "neighbor $neigh_addr" \
            $cli_out_for_neigh_addr dummy]} {

        #try ping
        set ping_result [ping_command $neigh_addr]

        if { $ping_result == 0 } {
            append msg_neighbor "wrong neighbor configuration. "
            append msg_neighbor "The neighbor address $neigh_addr "
            append msg_neighbor "might not exist. Plese check the correct "
            append msg_neighbor "one and reconfigure it"
        }
    }
   
    if { $verbose == 1 } {
	 puts "check_neighbor_problem: $msg_neighbor"
    }

    return $msg_neighbor
}

##############################################################################
#
#  Description: check_stuck_state
#               check if there is a stuck in the router by checking the
#               ospf status periodically at 23 secondes.
#               If it misses 2 hellos (70 seconds), it looks like a stuck
#               happens.
#
#  IN:          ospf state
#               state line output
#  OUT:         message for why it stucks.
#   
##############################################################################

proc check_stuck_state { state line } {

    global verbose
    global found_stuck
    set repeat_time 0

    set msg_stuck ""

    if [regexp $state $line dummy] {
	set stuck 0
	set neighbor_id [lindex $line 0]
        set neigh_addr [lindex $line 4]
        set intf [lindex $line 5]

	if { $state == "ATTEMPT" } {
	    if ![regexp "N/A" $neighbor_id dummy ] {
		return
	    }
	}

	while { $stuck < 3 } {
	    after 23000
	    incr repeat_time
	    if { $repeat_time == 10 } {
		if { $verbose == 1 } {
		    puts "Looks like there is no stuck at any state during"
	            puts "ospf formation"
		}
		break
	    }
		
            set old_stuck $stuck

            set cli_cmd "show ip ospf neighbor | include $state"
	    set cli_out [ios_enable_command $cli_cmd]
            foreach line_2 [split $cli_out "\r\n"] {
                if { [ regexp $state $line_2 ] &&
                [ regexp $neighbor_id $line_2 ] &&
                [ regexp $neigh_addr $line_2 ] } {
                    incr stuck
                    if { $verbose == 1 } {
                        puts "Stuck time check is $stuck"
                    }
                }
            }

            if { $old_stuck == $stuck } {
                break
            }
	}
	
	if { $stuck == 3 } {
            set found_stuck 1
	    if { $verbose == 1 } {
		puts "Stucks at $state"
	    }

	    if { $state == "ATTEMPT" } {
                set msg_stuck "Stuck at $state might be due to "
                set msg_neigh [check_neighbor_problem $neigh_addr]
                append msg_stuck $msg_neigh

                # mtu is also possible to stuck at ATTEMPT, at that case,
                # the neigh_addr is available.

                if { $msg_neigh == "" } {
                    set msg_stuck "OSPF neighbor $neigh_addr is stuck at "
                    append msg_stuck "$state might be due to "
                    append msg_mtu [check_mtu_problem $state $intf]
                    append msg_stuck $msg_mtu
                }
            } else {
                set msg_stuck "OSPF neighbor $neigh_addr is stuck at $state "
                append msg_stuck "might be due to "
                if { $state == "INIT" } {
                    append msg_stuck "access list on remote end "
                    append msg_stuck "blocking OSPF hellos or authentication "
                    append msg_stuck "config is present on one side, Please "
                    append msg_stuck "check the access-list or enable "
                    append msg_stuck "authentication on both sides"
                } elseif { $state == "EXSTART" } {
                    # stuck at EXSTART could be due to mtu problem
                    set msg_mtu [check_mtu_problem $state $intf]

                    # stuck at EXSTART also could be due to near end acl
                    set msg_stuck [check_acl_problem $intf $msg_mtu \
                            $msg_stuck]
                } elseif { $state == "EXCHANGE" } {
                    append msg_stuck [check_mtu_problem $state $intf]
                }
            }

	    if { $verbose == 1 } {
		puts "check_stuck_state $msg_stuck"
	    }
	}
    }
    return $msg_stuck
}

#Main function starts from here
global found_stuck
set found_stuck 0

set msg_header "\n\nTHE SERVICE DIAG MESSAGE FOR STUCK CASES is:"
set msg $msg_header

set cli_cmd "show ip ospf neighbor | exclude Neighbor ID"
set cli_out [ios_enable_command $cli_cmd]


foreach line [split $cli_out "\r\n"] {

    set state ""

    # Let us check if there is a stuck on the interface there
    if [regexp ATTEMPT $line] {
        set state ATTEMPT
    } elseif {[regexp INIT $line]} {
        set state INIT
    } elseif {[regexp EXSTART $line]} {
        set state EXSTART
    } elseif {[regexp EXCHANGE $line]} {
        set state EXCHANGE
    }

    if { $state != "" } {
        set stuck_msg [check_stuck_state $state $line]
        if { $stuck_msg != ""} {
            append msg \n\n $stuck_msg
        }
    }
}

if { $found_stuck == 0 } {
    return 
}

if { $msg == $msg_header } {
    append msg \n\n "reason unknown"
}

append msg \n\n "--------------------------------------------------------"
append msg "------"

action_syslog msg $msg

if { $send_email == 0 } {
    send_info $config_history $event_history 
} else {
    send_result_by_email $config_history $event_history $msg $_email_from \
       $_email_to $_email_cc $_email_server
}
