::cisco::eem::event_register_syslog occurs 1 pattern "\%OSPF-5-ADJCHG:.* from FULL to DOWN, Neighbor Down: Dead timer expired"  maxrun_sec 60000

#------------------------------------------------------------------
# EEM policy to report problem on ospf neighbor down. The error message can 
# appear at different cases. 
#
# May 2007 - Christine Yan (cyan@cisco.com)
#
# Copyright (c) 2007 by cisco Systems, Inc.
# All rights reserved.
#
# Version-date: v1.0-05/15/2007
#
# Name: ospf_timer_expired.tcl
#
# Desc:This script is an application to diagnose ospf problem. Report possible
#      ospf problem cased by "shut", "ip address list" or ospf configuration
#      removed.
# (1). Please note, when remote end in Fastethernet interface/gigabit 
#      interface, the syslog  message is not Dead timer expried.
#      Instead it is:
#      the syslog  message is not Dead timer expried. Instead it is 
#      "\%OSPF-5-ADJCHG:.*DOWN, Neighbor Down: Interface down or detached"
# (2). ospf is removed from the remote end.
# (3). ip access list is configured. 
#
#
# Environment Variables for this script are:
#  _send_email_for_timer_expired (opt)
#  _svcdiag_timerConfigHistory (opt)
#  _svcdiag_timerEventHistory  (opt)
#  _email_server (opt)
#  _email_from  (opt)
#  _email_to    (opt)
#  _email_cc    (opt)
#
# IOS config:
#
#  conf t
#
#  event manager environment _send_email_for_timer_expired no
#  event manager environment _svcdiag_timerConfigHistory FALSE
#  event manager environment _svcdiag_timerEventHistory FALSE
#  event manager directory user policy disk0:/svc_diag
#  event manager directory user library disk0:/user_lib
#  event manager policy ospf_timer_expired.tcl type user
#
#  event manager environment _email_server email.cisco.com
#  event manager environment _email_from cyan@cisco.com
#  event manager environment _email_to cyan@cisco.com
#  event manager environment _email_cc cyan@cisco.com
#
#  Please note: on some devices you do have to use
#  event manager directory user policy disk2:/
#
# For 3825 or 3xxx serial of routers, should use flash: instead of disk0:
#
# How to run this script
#
# As soon as the script is registered in config mode, it can be run since it 
# accepts a syslog event. 
# 
# so run the script by registering it:
# c7200-6(config)#event manamger policy ospf_timer_expired.tcl
#
#
# This script also needs a standard email format file to be present in 
# customer disk. The file name would be email_template_cmd.
# it should be present in :/user_lib. You need to copy the standard email
# template from tftp server to disk_name:/user_lib
#
#
#------------------------------------------------------------------
# The namespace import commands that follow import some cisco TCL extensions
# that are required to run this script

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

global verbose
set verbose 0

if {[info exists _verbose_for_timer_expired]} {

    if [regexp yes $_verbose_for_timer_expired] {

	set verbose 1

    }
}

# check if all the env variables we need exist
# If any of them doesn't exist, print out an error msg and quit
set send_email 0

if {[info exists _send_email_for_timer_expired]} {

    if [regexp yes $_send_email_for_timer_expired] {

	# Check if all the env variables we need exist
	# If any of them doesn't exist, print out an error msg and quit

	if {![info exists _email_server]} {
	    set result \
		"Policy cannot be run: variable _email_server has not been set"
	    error $result "_email_server is required for sending email"
	}
	if {![info exists _email_from]} {
	    set result \
		"Policy cannot be run: variable _email_from has not been set"
	    error $result "_email_from is required for sending email"
	}
	if {![info exists _email_to]} {
	    set result \
		"Policy cannot be run: variable _email_to has not been set"
	    error $result "_email_to is required for sending email"
	}
	if {![info exists _email_cc]} {
	    #_email_cc is an option, must set to empty string if not set.
	    set _email_cc ""
	}

	set send_email 1	
    }
}

set config_history 0

if {[info exists _svcdiag_timerConfigHistory]} {
    if { $_svcdiag_timerConfigHistory == "TRUE" } {
	set config_history 1
    }
}

set event_history 0

if {[info exists _svcdiag_timerEventHistory]} {
    if { $_svcdiag_timerEventHistory == "TRUE" } {
	set event_history 1
    }
}

array set arr_einfo [event_reqinfo]
set syslog_msg $arr_einfo(msg)
set r_msg ""

# Get the interface name from syslog message.
 
if {![ regexp {Nbr (.*) on (.*) from} $syslog_msg dummy neigh_addr intf ]} {
    # Check if the problem is caused by interface down/down. 
    return 
} else {
    if { $verbose == 1} {
        puts "Nbr is $neigh_addr, intf is $intf"
    }
}

set cli_cmd "show ip int brief | include $intf"
set cli_out [ios_enable_command $cli_cmd]

if { $verbose == 1} {
    puts "cli_out = $cli_out" 
}

set ios_cli "show run int $intf | i ip address"
set cli_ip_address_out [ios_enable_command $ios_cli]

if [regexp {ip address (.*) (.*)$} \
	$cli_ip_address_out dummy intf_addr subnet_mask] {

    if { $verbose == 1} {
        puts "ip address is $intf_addr \
		subnet_address is $subnet_mask"
    }

    regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} \
        $intf_addr dummy intf_ip(1) intf_ip(2) \
        intf_ip(3) intf_ip(4)
}

set msg_header "\n\nTHE SERVICE DIAG MESSAGE FOR TIMER EXPIRED is:"
set msg $msg_header

if [regexp {down                  down} $cli_out dummy] {

    append msg \n\n "The OSPF neighbor $neigh_addr interface $intf is in "
    append msg "down down state.  Needs to fix the interface down "
    append msg "problem first. Please check if the interface is "
    append msg "physically connected well "

    if [regexp -nocase {serial} $cli_out dummy] {
	# If the remote interface is ethernet, even it is shut, we do not get 
	# near end at down down
        append msg "or if the remote interface is shut down"
    }

} elseif [regexp {up (.*) down} $cli_out dummy] {

    append msg \n "OSPF neighbor $neigh_addr down might be due to "
    append msg "remote end shut down or other layer 2 interface protocol "
    append msg "problem"

} elseif [regexp {up (.*) up} $cli_out dummy] {

    if [regexp -nocase {serial} $intf dummy] {

	if { $verbose == 1 } {
	    puts "The physical interface should be connected well"
	}

    } elseif [regexp -nocase {ethernet} $intf dummy] {

        append msg \n "OSPF neighbor $neigh_addr down might be due to "

        set r_msg "remote end shut down, remote end OSPF configuration "
        append r_msg "removed, remote end has a passive interface, "

        # Check if the interface is in same subnet of this network
        # configuration. If it is, check its area id.
        # When near end has different area id as the remote end, the
        # interface can be down.

        set ospf_area_id [get_intf_area_id $intf_ip(1) $intf_ip(2) \
		$intf_ip(3) $intf_ip(4)]

        if { $ospf_area_id != "" } {
            append r_msg "remote end has a different area id (the near " 
            append r_msg "end now has an ospf area id as $ospf_area_id). "
        }
    }
}

set msg [check_acl_problem $intf $r_msg $msg]

if { $msg == $msg_header } {
    append msg \n\n "reason unknown"
}

append msg \n\n "--------------------------------------------------------"
append msg "------"

action_syslog msg $msg

if { $send_email == 0 } {
   send_info $config_history $event_history 
} else {
   send_result_by_email $config_history $event_history $msg $_email_from \
       $_email_to $_email_cc $_email_server
}
